<?php declare(strict_types=1);

namespace App\Controller;

use App\Entity\Lenses;
use App\Form\LensesType;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * Lens controller.
 */
#[Route(path: 'lens')]
class LensesController extends AbstractController
{
    use FormControllerTrait;

    protected const ENTITY = Lenses::class;
    protected const FORM   = LensesType::class;

    public function __construct(private readonly ManagerRegistry $managerRegistry)
    {
    }

    /**
     * Lists all lens entities.
     */
    #[Route(path: '/', name: 'lens_index', methods: ['GET'])]
    public function indexAction(): Response
    {
        $em            = $this->managerRegistry->getManager();
        $receivedItems = $em->getRepository(self::ENTITY)->findBy([
            'received' => true,
        ], [
            'brand'          => 'ASC',
            'productLine'    => 'ASC',
            'mount'          => 'ASC',
            'minFocalLength' => 'ASC',
            'maxFStop'       => 'ASC',
        ]);
        $newItems = $em->getRepository(self::ENTITY)->findBy([
            'received' => false,
        ], [
            'brand'          => 'ASC',
            'productLine'    => 'ASC',
            'mount'          => 'ASC',
            'minFocalLength' => 'ASC',
            'maxFStop'       => 'ASC',
        ]);

        return $this->render('lenses/index.html.twig', [
            'not_received' => $newItems,
            'lenses'       => $receivedItems,
        ]);
    }

    /**
     * Creates a new lens entity.
     */
    #[Route(path: '/new', name: 'lens_new', methods: ['GET', 'POST'])]
    public function newAction(Request $request): RedirectResponse|Response
    {
        return $this->itemCreate($request, 'lenses/new.html.twig', 'lense', 'lens_show');
    }

    /**
     * Finds and displays a lens entity.
     */
    #[Route(path: '/{id}', name: 'lens_show', methods: ['GET'])]
    public function showAction(Lenses $lens): Response
    {
        return $this->itemView($lens, 'lenses/show.html.twig', 'lense');
    }

    /**
     * Displays a form to edit an existing lens entity.
     */
    #[Route(path: '/{id}/edit', name: 'lens_edit', methods: ['GET', 'POST'])]
    public function editAction(Request $request, Lenses $lens): RedirectResponse|Response
    {
        return $this->itemUpdate($request, $lens, 'lenses/edit.html.twig', 'lense', 'lens_show');
    }

    /**
     * Moves a camera to the previouslyOwned table
     *
     * @param Request $request
     *
     * @return RedirectResponse
     */
    #[Route(path: '/{id}/deacquire', name: 'lens_deacquire', methods: ['POST'])]
    public function deacquireAction(Request $request, Lenses $lens): RedirectResponse
    {
        return $this->itemDeacquire($request, $lens, 'previously-owned-lens_index');
    }

    /**
     * Deletes a lens entity.
     */
    #[Route(path: '/{id}', name: 'lens_delete', methods: ['DELETE'])]
    public function deleteAction(Request $request, Lenses $lens): RedirectResponse
    {
        return $this->itemDelete($request, $lens, 'lens_index');
    }

    /**
     * Creates a form to delete a lens entity.
     *
     * @param Lenses $lens The lens entity
     *
     * @return FormInterface The form
     */
    private function createDeleteForm(Lenses $lens): FormInterface
    {
        return $this->buildForm($lens, 'lens_delete', 'DELETE');
    }

    /**
     * Creates a form to move
     *
     * @param Lenses $lens The lens entity
     */
    private function createDeacquireForm(Lenses $lens): FormInterface
    {
        return $this->buildForm($lens, 'lens_deacquire');
    }
}
